import React, {useEffect} from "react";
import {ActivityIndicator, Alert, FlatList, StyleSheet, View,} from "react-native";
import {RobotoBoldText} from "components/text/StyledText";

import AlephService from "services/AlephService";
import Item from "model/Item";
import {GLOBAL_STYLES} from "style/GlobalStyles";
import {StyleConstants} from "../style/StyleConstants";
import {Button} from "react-native-elements";


type Props = {
    docNumber: String,
    route: any
}

type State = {
    loading: boolean;
    items: Array<Item>,
    noResults: boolean,
    reservationInProgress: boolean
}

export const BookItemScreen: React.FC<Props> = (props: Props) => {

    const [itemsState, setState] = React.useState<State>({
        loading: true,
        items: [],
        noResults: false,
        reservationInProgress: false
    });

    const loadItems = async () => {
        const items = await AlephService.getItemData(props.route.params.docNumber);

        setState({
            ...itemsState,
            items: items,
            loading: false,
            reservationInProgress: false
        });
    };

    const flatListItemSeparator = () => {
        return (
            <View style={styles.itemSeparator}/>
        );
    };

    const listFooterComponent = () => {
        if (itemsState.loading) {
            return <ActivityIndicator style={GLOBAL_STYLES.indicator} size="small"
                                      color={StyleConstants.PRIMARY_BLUE_COLOR}/>
        } else {
            return null;
        }
    };

    useEffect(() => {
        loadItems();
    }, []);


    const renderItem = ({item}: { item: Item }) => (
        <View style={styles.mainContainer}>
            <View style={styles.container}>
                <View>
                    <RobotoBoldText style={styles.heading}>Typ výpůjčky:</RobotoBoldText>
                    <RobotoBoldText style={styles.heading}>Sbírka:</RobotoBoldText>
                    <RobotoBoldText style={styles.heading}>Signatura:</RobotoBoldText>
                    <RobotoBoldText style={styles.heading}>Studijní signatura:</RobotoBoldText>
                    <RobotoBoldText style={styles.heading}>Čárový kód:</RobotoBoldText>
                </View>
                <View style={styles.rightSection}>
                    <RobotoBoldText style={styles.itemInfo}>{item.itemStatus}</RobotoBoldText>
                    <RobotoBoldText style={styles.itemInfo}>{item.collection}</RobotoBoldText>

                    <RobotoBoldText style={styles.itemInfo}>{item.signature}</RobotoBoldText>

                    {/*pouze u prezencnich knih*/}
                    <RobotoBoldText style={styles.itemInfo}>{item.studySignature}</RobotoBoldText>
                    <RobotoBoldText style={styles.itemInfo}>{item.barCode}</RobotoBoldText>
                </View>
            </View>

            <View>
                <View style={{flexDirection: "row"}}>
                    <RobotoBoldText
                        style={[styles.statusLabel, {backgroundColor: item.loanDueDate != null ? "#F05E23" : "#00A86B",}]}>{item.state}</RobotoBoldText>

                    {item.loanDueDate != null &&
                    <Button
                        title={"Rezervovat"}
                        onPress={async () => {
                            try {
                                setState({
                                        ...itemsState,
                                        reservationInProgress: true
                                    }
                                );
                                await AlephService.holdRequest(item.docNumber, item.barCode)
                                setState({
                                        ...itemsState,
                                        reservationInProgress: false
                                    }
                                );
                                Alert.alert("Vytvoření rezervace", "Byl jste zařazen/a do fronty na vypůjčení této publikace. Jakmile na Vás přijde řada, budete informován/a e-mailem a budete mít 6 pracovních dnů na vyzvednutí rezervované publikace. Pokud si nestihnete rezervovanou publikaci vyzvednout nebo zrušit do 6 pracovních dnů, bude rezervace zrušena a budete penalizován/a pokutou 10 Kč.");
                            } catch (e) {
                                setState({
                                        ...itemsState,
                                        reservationInProgress: false
                                    }
                                );
                                Alert.alert("Vytvoření rezervace","Došlo k chybě při rezervaci položky");
                            }
                        }
                        }
                        buttonStyle={styles.button}
                        titleStyle={{fontSize: 12}}
                    />
                    }
                </View>

            </View>

        </View>
    );

    return (
        <View style={styles.list} >
            {itemsState.reservationInProgress && <ActivityIndicator style={GLOBAL_STYLES.fullScreenIndicator}/>}
            <FlatList<Item> data={itemsState.items} renderItem={renderItem} keyExtractor={item => item.barCode}
                            ItemSeparatorComponent={flatListItemSeparator} refreshing={itemsState.loading}
                            ListFooterComponent={listFooterComponent}/>
        </View>
    );

};

const styles = StyleSheet.create({
    list: {
        margin: 10,
    },
    mainContainer: {
        flexDirection: "column",
        backgroundColor: StyleConstants.BACKGROUND
    },
    container: {
        flexDirection: "row",
        backgroundColor: StyleConstants.BACKGROUND,
        margin: 4
    },
    rightSection: {
        marginLeft: 10
    },
    itemSeparator: {
        height: 10,
        width: '100%',
        // color: StyleConstants.PRIMARY_BLUE_COLOR,
        // backgroundColor: StyleConstants.PRIMARY_BLUE_COLOR,

    },
    heading: {
        color: '#7f7f7f'
    },
    itemInfo: {
        color: StyleConstants.PRIMARY_BLUE_COLOR,
    },
    button: {
        backgroundColor: "#00A86B",
        // backgroundColor: StyleConstants.PRIMARY_BLUE_COLOR,
        // alignSelf: "flex-end",
        // margin: 4
    },
    statusLabel: {
        color: "white", padding: 4, margin: 4,
    }
});